# ********************************************************************
# Copyright 2021 VMware, Inc.  All rights reserved. -- VMware Confidential
# ********************************************************************

"""httptask to simulate client to send http request to vadc, and parse
the response to extract the needed data
"""

import logging
import requests
import urllib3
import xml.etree.ElementTree as ET

from httptask import *
from request import Request

urllib3.disable_warnings()

# HttpTask base class
class HttpTask:
    OK_RESULT = "ok"
    ERROR_RESULT = "error"

    def __init__(self, sessionid, request, host='127.0.0.1'):
        self._jsessionid = sessionid
        self._request  = request
        self._response = None
        self._isAdminTask = False
        self._host = host

    def isValidResponse(self):
        if self._response is not None and self._response.status_code == 200:
            return True

        if self._response is None:
            logging.error("Response of %s is None" % self.__class__.__name__)
        else:
            logging.error("Error response code %d" % self._response.status_code)
        return False

    def processTask(self):
        url = "https://" + self._host + ":443/broker/xml"
        if self._isAdminTask:
            url = "https://" + self._host + ":8443/vadc/admin"

        cookie = "com.vmware.vdi.broker.location.id=cd832872f0dbfee827aebe053d768685e5de540f15e8259cf4bf9750e04a6c1b"
        if self._jsessionid is not None:
            cookie = "JSESSIONID=" + self._jsessionid + ";" + cookie

        self._response = requests.post(url,
              headers={'Content-Type':'application/xml',
                       'Cookie':cookie,
                       'User-Agent':'VMware-client'},
              data=self._request,
              verify=False)

        if self.isValidResponse():
            return self.parseResponse()

        return False

    def parseResponse():
        return False

    def getResponse(self):
        return self._response.text


# Admin DoListSessionTask class
class HttpDoListSessionTask(HttpTask):
    def __init__(self, host='127.0.0.1'):
        super().__init__(None, Request.admin_do_list_session(), host)
        self._session = dict()
        self._isAdminTask = True

    def isSessionStateExpected(self, connectionid, state):
        if connectionid in self._session and \
           self._session[connectionid]['state'] == state:
            return True

        if connectionid in self._session:
            logging.warning("Expected state %s, Actual state %s" %
                          (state, self._session[connectionid]['state']))
        else:
            logging.warning("No session found for connectionid %s" %
                            connectionid)

        return False

    def getAgentSessionId(self, connectionid):
        if connectionid in self._session:
            return self._session[connectionid]['sessionid']

        logging.error("No sessionid found for connectionid %s" % connectionid)
        return None

    def isSessionLogout(self, connectionid):
        if connectionid not in self._session:
            return True

        kits.Comment("Session is not logout, connectionid %s, state %s" %
                     (connectionid, self._session[connectionid]['state']))
        return False

    def parseResponse(self):
        response = self.getResponse()
        logging.debug("Response is %s" % response)

        root = ET.fromstring(response)
        result = root.find("list-session/result").text
        if result != HttpTask.OK_RESULT:
            logging.error("Request list-session failed, result %s" % result)
            return False

        # Save all session infos with key connectionid
        for session in root.iterfind('list-session/TERMINALRESPONSE/SESSION'):
            sessionDict = dict()

            sessionDict['username'] = session.find("USERNAME").text
            sessionDict['state'] = session.find("STATE").text
            sessionDict['sessionid'] = session.find("SESSIONID").text

            connectionid = session.find("CONNECTIONID").text
            self._session[connectionid] = sessionDict

            logging.debug("Saved username %s, connectionid %s, state %s" %
                         (sessionDict['username'],
                          connectionid, sessionDict['state']))

        return True

# Admin DoDisconnectSessionTask class
class HttpDoDisconnectSessionTask(HttpTask):
    def __init__(self, username, host='127.0.0.1'):
        super().__init__(None, Request.admin_do_disconnect_session(username), host)
        self._session = dict()
        self._isAdminTask = True

    def parseResponse(self):
        response = self.getResponse()
        logging.debug("Response is %s" % response)

        root = ET.fromstring(response)
        result = root.find("disconnect-session/result").text
        if result != HttpTask.OK_RESULT:
            logging.error("Request disconnect-session failed, result %s" % result)
            return False

        return True

# Admin DoKillSessionTask class
class HttpDoKillSessionTask(HttpTask):
    def __init__(self, username, host='127.0.0.1'):
        super().__init__(None, Request.admin_do_kill_session(username), host)
        self._session = dict()
        self._isAdminTask = True

    def parseResponse(self):
        response = self.getResponse()
        logging.debug("Response is %s" % response)

        root = ET.fromstring(response)
        result = root.find("kill-session/result").text
        if result != HttpTask.OK_RESULT:
            logging.error("Request kill-session failed, result %s" % result)
            return False

        return True
