#!/bin/sh

###################################################
# Copyright 2021-2022 VMware, Inc.  All rights reserved.
###################################################

# Constants
VADC_PORT=443
VADC_GROUP=vmwvadc
VADC_CFG_DIR=/etc/vmware/vadc
VADC_CFG_FILE=/etc/vmware/vadc/viewagent-vadc.conf

#
# Note, this usage doesn't consider already registered agents.
# prompt the usage and exit if user doesn't input mandatory parameters
#
usage_exit() {
   name=`basename $0`

   echo "Usage: $name [OPTION]... [VALUE]..."
   echo ""
   echo "Optional parameters but required information"
   echo "-A        yes|no <Accept|Refuse VMware General Terms and FIPS statement>"
   echo ""
   echo "Example:"
   echo "  sudo ./install_vadc.sh -A yes"
   echo ""

   exit 2
}

#
# Show EULA (End-user license agreement) to user
# If user accept the terms installaltion will continue
# or the installation will be terminated
#
verify_EULA_agreement()
{
    EULA_FILE=`dirname $0`/VMwareGeneralTerms.txt

    # Show EULA on terminal
    cat $EULA_FILE
    # Tell user to type [Yes/No]
    echo_term -n "Are you sure to install Linux Agent VADC (y/n)?"
    read USER_INPUT
    echo_term
    # check user input
    case "$USER_INPUT" in
        Y|y)
            # User accept the license agreement, installation continue
            echo_term "Installation starting..."
            ;;
        N|n)
            # User reject the license agreement, installtion terminate
            echo_term "Installation stopped."
            exit 2
            ;;
          *)
            # Invalid input
            echo_term "Invalid input and installation stopped."
            exit 2
            ;;
    esac
}

verify_linkclone() {
   vmware-rpctool "info-get guestinfo.ngaFork" >/dev/null 2>&1
   if [ "$?" != "0" ]; then
      return
   fi

   vmware-rpctool "info-get guestinfo.forked" >/dev/null 2>&1
   if [ "$?" != "0" ]; then
      return
   fi

   local ngaFork=`vmware-rpctool "info-get guestinfo.ngaFork"` >/dev/null 2>&1
   local forked=`vmware-rpctool "info-get guestinfo.forked"` >/dev/null 2>&1
   if [ "$ngaFork" = "1" -a "$forked" = "1" ]; then
      echo_term "ERROR: Linux Agent VADC doesn't support link clone VM."
      exit 1
   fi
}

verify_buildversion() {
   local agentBuildNumber=`awk -F '-' 'NR==1 {print $NF}'  ${VIEWAGENT_PATH}/Product.txt`
   local vadcBuildNumber=`awk -F '-' 'NR==1 {print $NF}' Product.txt`
   if [ $agentBuildNumber != $vadcBuildNumber ]; then
      local agentVersion=`sed -n '1p' ${VIEWAGENT_PATH}/Product.txt`
      local vadcVersion=`sed -n '1p' Product.txt`
      echo_term "ERROR: VADC version mismatches the Linux Agent version."
      echo_term "VADC version is $vadcVersion"
      echo_term "The Linux Agent version is $agentVersion"
      echo_term ""
      exit 1
   fi
}

verify_nginx() {
   which nginx > /dev/null 2>&1
   if [ "$?" != "0" ]; then
      echo_term "ERROR: Missing nginx service, please install nginx first."
      exit 1
   fi

   if [ ! -d "/etc/nginx/conf.d" ]; then
      echo_term "ERROR: Missing /etc/nginx/conf.d folder, please check or reinstall nginx."
      exit 1
   fi
}

verify_environment() {
   #
   # Installation location
   #
   if [ ! -d $VIEWAGENT_PATH ]; then
      echo_term "$VIEWAGENT_PATH not found"
      exit 1
   fi

   if [ ! -f ${VIEWAGENT_PATH}/Product.txt ]; then
      echo_term "ERROR: Linux View Agent isn't installed."
      echo_term "Please install Linux View Agent before install Linux VADC."
      echo_term ""
      exit 1
   fi

   verify_linkclone
   verify_buildversion
   verify_nginx
}

extract_vadc() {
   local xmlapi_path=${VIEWAGENT_PATH}/vadc

   tar xfChp VMware-viewagent-vadc-*.tar $VIEWAGENT_JAVA_PATH
   if [ "$?" != "0" ]; then
      echo_term "Failed to extract Linux Agent VADC package."
      exit 1
   fi

   chown root:root ${VIEWAGENT_JAVA_PATH}/*.jar
   chmod 644 ${VIEWAGENT_JAVA_PATH}/*.jar

   if [ ! -d $xmlapi_path ]; then
      mkdir -p $xmlapi_path
   fi

   cp -p plugin/libxmlapi.so $xmlapi_path
   chown -R root:root $xmlapi_path
   chmod -R 755 $xmlapi_path

   # Copy tomcat_selinux.sh
   cp -f tomcat_selinux.sh ${VIEWAGENT_PATH}/vadc/
   chown root:root ${VIEWAGENT_PATH}/vadc/tomcat_selinux.sh
   chmod 750 ${VIEWAGENT_PATH}/vadc/tomcat_selinux.sh

   cp -f uninstall_vadc.sh ${VIEWAGENT_PATH}/bin/
   chown root:root ${VIEWAGENT_PATH}/bin/uninstall_vadc.sh
   chmod 750 ${VIEWAGENT_PATH}/bin/uninstall_vadc.sh

   if [ ! -d "/etc/vmware/vadc" ]; then
      mkdir -p /etc/vmware/vadc
      chown root:root /etc/vmware/vadc
      chmod 755 /etc/vmware/vadc
   fi

   # If FIPS is enable, we use fips bouncycastle library.
   grep -i "^FIPS_ENABLE=true" ${AGENT_CFG_FILE} >/dev/null 2>&1
   if [ "$?" = "0" ]; then
      rm -f $VIEWAGENT_JAVA_PATH/bcprov-jdk15on.jar
      rm -f $VIEWAGENT_JAVA_PATH/bcpkix-jdk15on.jar
      rm -f $VIEWAGENT_JAVA_PATH/bctls-jdk15on.jar
      rm -f $VIEWAGENT_JAVA_PATH/bcutil-jdk15on.jar
   else
      rm -f $VIEWAGENT_JAVA_PATH/bc-fips.jar
      rm -f $VIEWAGENT_JAVA_PATH/bcpkix-fips.jar
      rm -f $VIEWAGENT_JAVA_PATH/bctls-fips.jar
   fi
}

configure_vadc() {
   grep "^VADCINST" $AGENT_CFG_FILE >/dev/null 2>&1
   if [ "$?" = "0" ]; then
      sed -i '/^VADCINST/d' $AGENT_CFG_FILE >/dev/null 2>&1
   fi
   echo "VADCINST=true" >> $AGENT_CFG_FILE

   # Create the vmware vadc group
   if ! getent group $VADC_GROUP >/dev/null 2>&1; then
      echo_term "Begin to create vmware vadc group $VADC_GROUP"
      ${ADDGROUP} --system $VADC_GROUP > /dev/null 2>&1
   fi

   # Copy vadc config template. If template exists, overwrite it
   cp -f template/viewagent-vadc.conf.template ${VADC_CFG_FILE}.template
   chown root:root ${VADC_CFG_FILE}.template
   chmod 644 ${VADC_CFG_FILE}.template

   # Copy out vadc config file when fresh install
   if [ ! -f "$VADC_CFG_FILE" ]; then
      cp template/viewagent-vadc.conf.template $VADC_CFG_FILE
      chown root:root $VADC_CFG_FILE
      chmod 644 $VADC_CFG_FILE
   fi

   grep -i "^SINGLETON_MODE=false" $AGENT_CFG_FILE >/dev/null 2>&1
   if [ "$?" = "0" ]; then
      if [ ! -d "${VADC_CFG_DIR}/published_apps" ]; then
         mkdir -p ${VADC_CFG_DIR}/published_apps
         chown root:root ${VADC_CFG_DIR}/published_apps
         chmod 755 ${VADC_CFG_DIR}/published_apps
      fi

      cp -f template/PublishedApp.template ${VADC_CFG_DIR}/published_apps
      chown root:root ${VADC_CFG_DIR}/published_apps/PublishedApp.template
      chmod 755 ${VADC_CFG_DIR}/published_apps/PublishedApp.template
   fi

   grep -i "^FIPS_ENABLE=false" $AGENT_CFG_FILE >/dev/null 2>&1
   if [ "$?" = "0" ]; then
      if [ -f "$JRE_PATH/conf/security/java.security" ]; then
         sed -i 's/^securerandom.source=file:\/dev\/random/securerandom.source=file:\/dev\/urandom/g' \
            $JRE_PATH/conf/security/java.security
      fi
   fi
}

configure_nginx() {
   if [ -d "/etc/nginx/conf.d" ]; then
      # Place vmwvadc.conf only when fresh install.
      if [ ! -f "/etc/nginx/conf.d/vmwvadc.conf" ]; then
         cp -f nginx/vmwvadc.conf /etc/nginx/conf.d
         chown root:root /etc/nginx/conf.d/vmwvadc.conf
         chmod 644 /etc/nginx/conf.d/vmwvadc.conf

         echo_term "Begin to restart nginx service..."
         service nginx restart
      fi

      systemctl is-enabled nginx | grep -q enabled || \
      systemctl enable nginx >/dev/null 2>&1
   else
      echo_log "ERROR: Missing /etc/nginx/conf.d/vmwvadc.conf."
   fi
}

configure_nginx_policy() {
   local tmpDir=$VIEWAGENT_PATH/selinux_nginx.tmp
   local moduleName=vmwnginx
   local policyContent=$tmpDir/$moduleName.te
   local binaryModule=$tmpDir/$moduleName.mod
   local modulePackage=$tmpDir/$moduleName.pp

   mkdir -p $tmpDir
   chmod -R 700 $tmpDir

   echo "module $moduleName 1.0;" > $policyContent
   echo "require {" >> $policyContent
   echo "   type http_port_t;" >> $policyContent
   echo "   type http_cache_port_t;" >> $policyContent
   echo "   type pki_ca_port_t;" >> $policyContent
   echo "   type httpd_t;" >> $policyContent
   echo "   class tcp_socket name_connect;" >> $policyContent
   echo "}" >> $policyContent
   echo "allow httpd_t http_port_t:tcp_socket name_connect;" >> $policyContent
   echo "allow httpd_t pki_ca_port_t:tcp_socket name_connect;" >> $policyContent
   echo "allow httpd_t http_cache_port_t:tcp_socket name_connect;" >> $policyContent

   checkmodule -M -m $policyContent -o $binaryModule > /dev/null 2>&1
   semodule_package -m $binaryModule -o $modulePackage > /dev/null 2>&1
   semodule -i $modulePackage > /dev/null 2>&1

   [ -n "$tmpDir" ] && rm -rf $tmpDir
}

configure_selinux() {
   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})

      echo_term "Begin to configure selinux..."

      which checkmodule >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo_term "The required command \"checkmodule\" is currently not installed, configure selinux failed."
         return
      fi

      which semodule_package >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo_term "The required command \"semodule_package\" is currently not installed, configure selinux failed."
         return
      fi

      which semodule >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo_term "The required command \"semodule\" is currently not installed, configure selinux failed."
         return
      fi

      configure_nginx_policy
   ;;
   esac
}

#
# Include common constants and routines
#
. "`dirname $0`/commonlib.sh"

#
# Initialise command
#
define_os_version
define_commands
identify_distribution
define_constants
configure_distribution_common

SHOW_EULA="yes"

while getopts A: o
do
   case "$o" in
   A)
      #Accept EULA
      param="$OPTARG"
      case "$param" in
         yes)
               SHOW_EULA="no"
               ;;
          no)
               usage_exit
               ;;
           *)
               usage_exit
               ;;
      esac
      ;;
   esac
done

# Check if need to show EULA to End-user
if [ "$SHOW_EULA" = "yes" ]; then
   verify_EULA_agreement
fi

echo_term "Begin to install Linux Agent VADC..."

echo_term "Verify installation environment..."
verify_environment

echo_term "Extract VADC package..."
extract_vadc

echo_term "Configure VADC environment..."
configure_vadc

configure_selinux

echo_term "Configure nginx service..."
configure_nginx

echo_term "Enable ${VADC_PORT} port in firewall..."
configure_firewall ${VADC_PORT}

echo_term "Restart viewagent service..."
serviceRestart viewagent

echo_term "Finish to install Linux Agent VADC."
